@@ -1,7 +1,7 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * cairo-compmgr
- * Copyright (C) Nicolas Bruguier 2007 <gandalfn@club-internet.fr>
+ * Copyright (C) Nicolas Bruguier 2007-2010 <gandalfn@club-internet.fr>
  * 
  * cairo-compmgr is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,28 +24,18 @@
 #include "ccm.h"
 
 GType
-ccm_window_backend_get_type(CCMScreen* screen)
+ccm_window_backend_get_type (CCMScreen * screen)
 {
-	gchar* backend;
+    const gchar *backend = ccm_screen_get_backend(screen);
 #ifndef DISABLE_XRENDER_BACKEND
-	GType type = ccm_window_xrender_get_type();
+    GType type = ccm_window_xrender_get_type ();
 #else
-	GType type = 0;
+    GType type = 0;
 #endif
-	
-	g_object_get(G_OBJECT(screen), "backend", &backend, NULL);
-	if (backend)
-	{
-#ifndef DISABLE_GLITZ_BACKEND
-		if (!g_ascii_strcasecmp(backend, "glitz"))
-			type = ccm_window_glitz_get_type();
-#endif
-#ifndef DISABLE_OPENVG_BACKEND
-		if (!g_ascii_strcasecmp(backend, "openvg"))
-			type = ccm_window_openvg_get_type();
-#endif
-		g_free(backend);
-	}
-	
-	return type;
+
+    if (backend)
+    {
+    }
+
+    return type;
 }
